/*
 * FILE				: gui_core.c
 * DESCRIPTION		: This file is main files.
 * Author			: XiaomaGee@Gmail.com
 * Copyright		:
 *
 * History
 * --------------------
 * Rev	: 0.00
 * Date	: 03/05/2012
 *
 * create.
 * --------------------
 */

//---------------- Include files ------------------------//
#include "..\include\tft.h"
#include "..\include\gui_core.h"
#include "..\include\hardware.h"
#include "..\fatfs\ff.h"
#include "..\include\systick.h"
#include "..\include\font.h"

#include <stdarg.h>
#include <math.h>
#include <stdlib.h>

//---------------- Function Prototype -------------------//
static int dot(int x, int y, COLOR_T c);
static int line(int /* x0 */, int /* y0 */, int /* x1 */, int /* y1 */, COLOR_T);
static int rect(RECT_T *);
static int box(BOX_T *);
static int line_to(int /* x */, int /* y */, COLOR_T /* c */);
static int set_cursor(int /* x */, int /* y */);
static int draw_arrow(int x, int y, int dir, COLOR_T c);
static int processbar(PROCESSBAR_T *);
static int splitter(SPLITTER_T *);
static int check_box(CHECK_BOX_T *);
static int circle(int x, int y, int r, int fill, COLOR_T c);
static int scrollbar(SCROLLBAR_T *);
static int show_picture(int x, int y, int width, int height, char * path);

//---------------- Variable -----------------------------//
GUI_T gui = {
	.dot = dot,
	.line = line,
	.rect = rect,
	.box = box,
	.arrow = draw_arrow,
	.set_cursor = set_cursor,
	.line_to = line_to,
	.processbar = processbar,
	.splitter = splitter,
	.check_box = check_box,
	.circle = circle,
	.scrollbar = scrollbar,
	.show_picture = show_picture,
	.region.x0 = 0,
	.region.y0 = 0,
	.region.x1 = LCD_XSIZE,
	.region.y1 = LCD_YSIZE
};

static int cursor_x = 0;
static int cursor_y = 0;

//-----------------Function------------------------------//
/*
 * Name				: line
 * Description		: ---
 * Author			: XiaomaGee.
 *
 * History
 * ----------------------
 * Rev	: 0.00
 * Date	: 03/08/2012
 *
 * create.
 * ----------------------
 */
static int
line(int x1, int y1, int x2, int y2, COLOR_T c)
{
	int dx = x2 - x1;
	int dy = y2 - y1;
	int ux = ((dx > 0) << 1) - 1; //xȡ-1
	int uy = ((dy > 0) << 1) - 1; //yȡ-1
	int x = x1, y = y1, eps;          //epsΪۼ

	int w = x2 - x1;
	int h = y2 - y1;
	int i;

	int absw = abs(w);
	int absh = abs(h);

	if (h == 0) {
		lcd_ctl.b.csrmode = CSRMODE_H;
		lcd_ctl(lcd_ctl.w);

		lcd_csrx(x2 > x1 ? x1 : x2);
		lcd_csry(y1);

		for (i = 0; i < absw + 1; i++) lcd_data(c);

		return 0;
	}

	if (w == 0) {
		lcd_csrx(x1);
		lcd_csry(y2 > y1 ? y1 : y2);

		lcd_ctl.b.csrmode = CSRMODE_V;
		lcd_ctl(lcd_ctl.w);

		for (i = 0; i < absh + 1; i++) lcd_data(c);

		return 0;
	}

	eps = 0; dx = abs(dx); dy = abs(dy);
	if (dx > dy) {
		for (x = x1; x != x2; x += ux) {
			gui.dot(x, y, c);
			eps += dy;
			if ((eps << 1) >= dx) {
				y += uy; eps -= dx;
			}
		}
	}else {
		for (y = y1; y != y2; y += uy) {
			dot(x, y, c);
			eps += dx;
			if ((eps << 1) >= dy) {
				x += ux; eps -= dy;
			}
		}
	}

	return 0;
}

/*
    int  w,h,k,i,flagx=0,flagy=0,absw,absh;

     w=x1-x0;
     h=y1-y0;

     absw=abs(w);
     absh=abs(h);

     if(x1>x0)flagx=1;
             else flagx=-1;

     if(y1>y0)flagy=1;
             else flagy=-1;

     if(h==0){
             lcd_ctl.b.csrmode=CSRMODE_H;
             lcd_ctl(lcd_ctl.w);

             lcd_csrx(x1>x0?x0:x1);
             lcd_csry(y0);

             for(i=0;i<absw+1;i++)lcd_data(c);

             return 0;
     }

     if(w==0){

             lcd_csrx(x0);
             lcd_csry(y1>y0?y0:y1);

             lcd_ctl.b.csrmode=CSRMODE_V;
             lcd_ctl(lcd_ctl.w);

             for(i=0;i<absh+1;i++)lcd_data(c);

             return 0;
     }

     if(absw>absh){

             k=absh*1000000;
             k/=absw;
             k*=flagy;

             for(i=0;i<absw;i++){
                     lcd_csrx(i*flagx+x0);
                     lcd_csry((k*i)/1000000+y0);
                     lcd_data(c);
             }
     }else{
              k=absw*1000000;
              k/=absh;
              k*=flagx;

             for(i=0;i<absh;i++){
                     lcd_csrx((k*i)/1000000+x0);
                     lcd_csry(y0+i*flagy);
                     lcd_data(c);
             }
     }

     return 0;

   }

 */

/*
 * Name				: rect
 * Description		: ---
 * Author			: XiaomaGee.
 *
 * History
 * ----------------------
 * Rev	: 0.00
 * Date	: 03/08/2012
 *
 * create.
 * ----------------------
 */
static int
rect(RECT_T * r)
{
	int i, j;

	if (r->fill_flag == 2) {
		lcd_ctl.b.csrmode = 0;
		lcd_ctl(lcd_ctl.w);

		for (j = 0; j < r->height; j++) {
			for (i = 0; i < r->width; i++) gui.dot(r->x + i, r->y + j, *r->p++);
		}
	}else if (r->fill_flag == 1) {
		if (r->height > r->width) {
			lcd_ctl.b.csrmode = CSRMODE_V;
			lcd_ctl(lcd_ctl.w);
			for (i = 0; i < r->width; i++) {
				lcd_csrx(i + r->x);
				lcd_csry(r->y);
				for (j = 0; j < r->height; j++) {
					lcd_data(r->color);
				}
			}
		}else{
			lcd_ctl.b.csrmode = CSRMODE_H;
			lcd_ctl(lcd_ctl.w);

			for (i = 0; i < r->height; i++) {
				lcd_csrx(r->x);
				lcd_csry(r->y + i);
				for (j = 0; j < r->width; j++) {
					lcd_data(r->color);
				}
			}
		}
	}else { //no fill
		lcd_ctl.b.csrmode = CSRMODE_H;
		lcd_ctl(lcd_ctl.w);

		lcd_csrx(r->x);
		lcd_csry(r->y);
		for (i = 0; i < r->width; i++) lcd_data(r->color);

		lcd_csrx(r->x);
		lcd_csry(r->y + r->height);
		for (i = 0; i < r->width; i++) lcd_data(r->color);

		lcd_ctl.b.csrmode = CSRMODE_V;
		lcd_ctl(lcd_ctl.w);

		lcd_csrx(r->x);
		lcd_csry(r->y);
		for (i = 0; i < r->height; i++) lcd_data(r->color);

		lcd_csrx(r->x + r->width);
		lcd_csry(r->y);
		for (i = 0; i < r->height + 1; i++) lcd_data(r->color);
	}

	return 0;
}

/*
 * Name				: box
 * Description		: ---
 * Author			: XiaomaGee.
 *
 * History
 * ----------------------
 * Rev	: 0.00
 * Date	: 03/08/2012
 *
 * create.
 * ----------------------
 */
static int
box(BOX_T * b)
{
	RECT_T r;
	unsigned short int buf[4][4] = {
		{
			COLOR_WINDOW_BACKGROUND, COLOR_WINDOW_BACKGROUND, COLOR_WINDOW_BACKGROUND, COLOR_WINDOW_BACKGROUND
		}, {
			COLOR_WINDOW_BACKGROUND, COLOR_WINDOW_BACKGROUND, COLOR_WINDOW_BACKGROUND, COLOR_WINDOW_BACKGROUND
		}, {
			COLOR_WINDOW_BACKGROUND, COLOR_WINDOW_BACKGROUND, COLOR_WINDOW_BACKGROUND, COLOR_WINDOW_BACKGROUND
		}, {
			COLOR_WINDOW_BACKGROUND, COLOR_WINDOW_BACKGROUND, COLOR_WINDOW_BACKGROUND, COLOR_WINDOW_BACKGROUND
		}
	};

	r.x = b->startx + 1;
	r.y = b->starty + 1;
	r.height = b->endy - b->starty - 1;
	r.width = b->endx - b->startx - 2;
	r.color = b->color;
	r.fill_flag = 1;
	gui.rect(&r);


	switch (b->attrib & 0x3) {
	case BOX_FRAME:

		b->endx -= 1;
		b->endy -= 1;

		gui.line(b->startx, b->starty, b->endx, b->starty, COLOR_GRAY);
		gui.line(b->startx, b->starty, b->startx, b->endy, COLOR_GRAY);
		gui.line(b->startx, b->endy, b->endx, b->endy, COLOR_GRAY);
		gui.line(b->endx, b->starty, b->endx, b->endy, COLOR_GRAY);

		gui.line(b->startx + 1, b->starty + 1, b->endx - 1, b->starty + 1, COLOR_WHITE);
		gui.line(b->startx + 1, b->starty + 1, b->startx + 1, b->endy - 1, COLOR_WHITE);
		gui.line(b->startx, b->endy + 1, b->endx + 1, b->endy + 1, COLOR_WHITE);
		gui.line(b->endx + 1, b->starty, b->endx + 1, b->endy + 1, COLOR_WHITE);

		break;

	case BOX_RECESSED:

		gui.line(b->startx, b->starty, b->endx - 1, b->starty, COLOR_GRAY);
		gui.line(b->startx, b->starty, b->startx, b->endy - 1, COLOR_GRAY);
		gui.line(b->startx, b->endy, b->endx - 1, b->endy, COLOR_WHITE);
		gui.line(b->endx - 1, b->starty, b->endx - 1, b->endy - 1, COLOR_WHITE);

		if (b->attrib & BOX_CIRCULAR_LEFT_UP) {
			buf[0][0] = COLOR_WINDOW_BACKGROUND;
			buf[0][1] = COLOR_WINDOW_BACKGROUND;
			buf[0][2] = COLOR_WINDOW_BACKGROUND;
			buf[0][3] = COLOR_WINDOW_BACKGROUND;

			buf[1][0] = COLOR_WINDOW_BACKGROUND;
			buf[1][1] = COLOR_WINDOW_BACKGROUND;
			buf[2][0] = COLOR_WINDOW_BACKGROUND;
			buf[3][0] = COLOR_WINDOW_BACKGROUND;

			buf[1][2] = COLOR_GRAY;
			buf[1][3] = COLOR_GRAY;
			buf[2][1] = COLOR_GRAY;
			buf[3][1] = COLOR_GRAY;

			buf[2][2] = b->color;
			buf[2][3] = b->color;
			buf[3][2] = b->color;
			buf[3][3] = b->color;

			r.x = b->startx;
			r.y = b->starty;
			r.height = 4;
			r.width = 4;
			r.fill_flag = 2;
			r.p = buf[0];

			gui.rect(&r);
		}

		if (b->attrib & BOX_CIRCULAR_LEFT_DOWN) {
			buf[3][0] = COLOR_WINDOW_BACKGROUND;
			buf[3][1] = COLOR_WINDOW_BACKGROUND;
			buf[3][2] = COLOR_WINDOW_BACKGROUND;
			buf[3][3] = COLOR_WINDOW_BACKGROUND;

			buf[0][0] = COLOR_WINDOW_BACKGROUND;
			buf[1][0] = COLOR_WINDOW_BACKGROUND;
			buf[2][0] = COLOR_WINDOW_BACKGROUND;
			buf[2][1] = COLOR_WINDOW_BACKGROUND;

			buf[2][2] = COLOR_WHITE;
			buf[2][3] = COLOR_WHITE;
			buf[0][1] = COLOR_WHITE;
			buf[1][1] = COLOR_WHITE;

			buf[0][2] = b->color;
			buf[0][3] = b->color;
			buf[1][2] = b->color;
			buf[1][3] = b->color;

			r.x = b->startx;
			r.y = b->endy - 3;
			r.height = 4;
			r.width = 4;
			r.fill_flag = 2;
			r.p = buf[0];

			gui.rect(&r);
		}
		if (b->attrib & BOX_CIRCULAR_RIGHT_UP) {
			buf[0][0] = COLOR_WINDOW_BACKGROUND;
			buf[0][1] = COLOR_WINDOW_BACKGROUND;
			buf[0][2] = COLOR_WINDOW_BACKGROUND;
			buf[0][3] = COLOR_WINDOW_BACKGROUND;

			buf[1][3] = COLOR_WINDOW_BACKGROUND;
			buf[2][3] = COLOR_WINDOW_BACKGROUND;
			buf[3][3] = COLOR_WINDOW_BACKGROUND;
			buf[1][2] = COLOR_WINDOW_BACKGROUND;

			buf[1][0] = COLOR_WHITE;
			buf[1][1] = COLOR_WHITE;
			buf[2][2] = COLOR_WHITE;
			buf[3][2] = COLOR_WHITE;

			buf[2][0] = b->color;
			buf[2][1] = b->color;
			buf[3][0] = b->color;
			buf[3][1] = b->color;

			r.x = b->endx - 4;
			r.y = b->starty;
			r.height = 4;
			r.width = 4;
			r.fill_flag = 2;
			r.p = buf[0];

			gui.rect(&r);
		}
		if (b->attrib & BOX_CIRCULAR_RIGHT_DOWN) {
			buf[0][3] = COLOR_WINDOW_BACKGROUND;
			buf[1][3] = COLOR_WINDOW_BACKGROUND;
			buf[2][3] = COLOR_WINDOW_BACKGROUND;
			buf[3][3] = COLOR_WINDOW_BACKGROUND;

			buf[3][0] = COLOR_WINDOW_BACKGROUND;
			buf[3][1] = COLOR_WINDOW_BACKGROUND;
			buf[3][2] = COLOR_WINDOW_BACKGROUND;
			buf[2][2] = COLOR_WINDOW_BACKGROUND;

			buf[0][2] = COLOR_WHITE;
			buf[1][2] = COLOR_WHITE;
			buf[2][0] = COLOR_WHITE;
			buf[2][1] = COLOR_WHITE;

			buf[0][0] = b->color;
			buf[0][1] = b->color;
			buf[1][0] = b->color;
			buf[1][1] = b->color;

			r.x = b->endx - 4;
			r.y = b->endy - 3;
			r.height = 4;
			r.width = 4;
			r.fill_flag = 2;
			r.p = buf[0];

			gui.rect(&r);
		}
		break;

	case BOX_RAISED:

		gui.line(b->startx, b->starty, b->endx - 1, b->starty, COLOR_WHITE);
		gui.line(b->startx, b->starty, b->startx, b->endy - 1, COLOR_WHITE);
		gui.line(b->startx, b->endy, b->endx - 1, b->endy, COLOR_GRAY);
		gui.line(b->endx - 1, b->starty, b->endx - 1, b->endy - 1, COLOR_GRAY);

		if (b->attrib & BOX_CIRCULAR_LEFT_UP) {
			buf[0][0] = COLOR_WINDOW_BACKGROUND;
			buf[0][1] = COLOR_WINDOW_BACKGROUND;
			buf[0][2] = COLOR_WINDOW_BACKGROUND;
			buf[0][3] = COLOR_WINDOW_BACKGROUND;

			buf[1][0] = COLOR_WINDOW_BACKGROUND;
			buf[1][1] = COLOR_WINDOW_BACKGROUND;
			buf[2][0] = COLOR_WINDOW_BACKGROUND;
			buf[3][0] = COLOR_WINDOW_BACKGROUND;

			buf[1][2] = COLOR_WHITE;
			buf[1][3] = COLOR_WHITE;
			buf[2][1] = COLOR_WHITE;
			buf[3][1] = COLOR_WHITE;

			buf[2][2] = b->color;
			buf[2][3] = b->color;
			buf[3][2] = b->color;
			buf[3][3] = b->color;

			r.x = b->startx;
			r.y = b->starty;
			r.height = 4;
			r.width = 4;
			r.fill_flag = 2;
			r.p = buf[0];

			gui.rect(&r);
		}

		if (b->attrib & BOX_CIRCULAR_LEFT_DOWN) {
			buf[3][0] = COLOR_WINDOW_BACKGROUND;
			buf[3][1] = COLOR_WINDOW_BACKGROUND;
			buf[3][2] = COLOR_WINDOW_BACKGROUND;
			buf[3][3] = COLOR_WINDOW_BACKGROUND;

			buf[0][0] = COLOR_WINDOW_BACKGROUND;
			buf[1][0] = COLOR_WINDOW_BACKGROUND;
			buf[2][0] = COLOR_WINDOW_BACKGROUND;
			buf[2][1] = COLOR_WINDOW_BACKGROUND;

			buf[2][2] = COLOR_WHITE;
			buf[2][3] = COLOR_WHITE;
			buf[0][1] = COLOR_WHITE;
			buf[1][1] = COLOR_WHITE;

			buf[0][2] = b->color;
			buf[0][3] = b->color;
			buf[1][2] = b->color;
			buf[1][3] = b->color;

			r.x = b->startx;
			r.y = b->endy - 3;
			r.height = 4;
			r.width = 4;
			r.fill_flag = 2;
			r.p = buf[0];

			gui.rect(&r);
		}
		if (b->attrib & BOX_CIRCULAR_RIGHT_UP) {
			buf[0][0] = COLOR_WINDOW_BACKGROUND;
			buf[0][1] = COLOR_WINDOW_BACKGROUND;
			buf[0][2] = COLOR_WINDOW_BACKGROUND;
			buf[0][3] = COLOR_WINDOW_BACKGROUND;

			buf[1][3] = COLOR_WINDOW_BACKGROUND;
			buf[2][3] = COLOR_WINDOW_BACKGROUND;
			buf[3][3] = COLOR_WINDOW_BACKGROUND;
			buf[1][2] = COLOR_WINDOW_BACKGROUND;

			buf[1][0] = COLOR_WHITE;
			buf[1][1] = COLOR_WHITE;
			buf[2][2] = COLOR_WHITE;
			buf[3][2] = COLOR_WHITE;

			buf[2][0] = b->color;
			buf[2][1] = b->color;
			buf[3][0] = b->color;
			buf[3][1] = b->color;

			r.x = b->endx - 4;
			r.y = b->starty;
			r.height = 4;
			r.width = 4;
			r.fill_flag = 2;
			r.p = buf[0];

			gui.rect(&r);
		}
		if (b->attrib & BOX_CIRCULAR_RIGHT_DOWN) {
			buf[0][3] = COLOR_WINDOW_BACKGROUND;
			buf[1][3] = COLOR_WINDOW_BACKGROUND;
			buf[2][3] = COLOR_WINDOW_BACKGROUND;
			buf[3][3] = COLOR_WINDOW_BACKGROUND;

			buf[3][0] = COLOR_WINDOW_BACKGROUND;
			buf[3][1] = COLOR_WINDOW_BACKGROUND;
			buf[3][2] = COLOR_WINDOW_BACKGROUND;
			buf[2][2] = COLOR_WINDOW_BACKGROUND;

			buf[0][2] = COLOR_GRAY;
			buf[1][2] = COLOR_GRAY;
			buf[2][0] = COLOR_GRAY;
			buf[2][1] = COLOR_GRAY;

			buf[0][0] = b->color;
			buf[0][1] = b->color;
			buf[1][0] = b->color;
			buf[1][1] = b->color;

			r.x = b->endx - 4;
			r.y = b->endy - 3;
			r.height = 4;
			r.width = 4;
			r.fill_flag = 2;
			r.p = buf[0];

			gui.rect(&r);
		}
		break;
	}

	return 0;
}

/*
 * Name				: set_cursor
 * Description		: ---
 * Author			: XiaomaGee.
 *
 * History
 * ----------------------
 * Rev	: 0.00
 * Date	: 03/08/2012
 *
 * create.
 * ----------------------
 */
static int
set_cursor(int x, int y)
{
	cursor_x = x;
	cursor_y = y;

	return 0;
}

/*
 * Name				: line_to
 * Description		: ---
 * Author			: XiaomaGee.
 *
 * History
 * ----------------------
 * Rev	: 0.00
 * Date	: 03/08/2012
 *
 * create.
 * ----------------------
 */
static int
line_to(int x, int y, COLOR_T c)
{
	line(cursor_x, cursor_y, x, y, c);
	set_cursor(x, y);

	return 0;
}

/*
 * Name				: dot
 * Description		: ---
 * Author			: XiaomaGee.
 *
 * History
 * ----------------------
 * Rev	: 0.00
 * Date	: 03/08/2012
 *
 * create.
 * ----------------------
 */
static int
dot(int x, int y, COLOR_T c)
{
	lcd_csrx(x);
	lcd_csry(y);
	lcd_data(c);

	return 0;
}

/*
 * Name				: draw_arrow
 * Description		: ---
 * Author			: XiaomaGee.
 *
 * History
 * ----------------------
 * Rev	: 0.00
 * Date	: 03/08/2012
 *
 * create.
 * ----------------------
 */
static int draw_arrow(int x, int y, int dir, COLOR_T c)
{
	int i;

	switch (dir) {
	case 0:
		for (i = 0; i <= 10; i++) gui.line(x, y, x - 5 + i, y + 15, c);
		break;

	case 1:
		for (i = 0; i <= 10; i++) gui.line(x, y, x - 15, y - 5 + i, c);
		break;

	case 2:
		for (i = 0; i <= 10; i++) gui.line(x, y, x - 5 + i, y - 15, c);
		break;
	case 3:
		for (i = 0; i <= 10; i++) gui.line(x, y, x + 15, y - 5 + i, c);
		break;
	}

	return 0;
}

/*
 * Name				: processbar
 * Description		: ---
 * Author			: XiaomaGee.
 *
 * History
 * ----------------------
 * Rev	: 0.00
 * Date	: 03/08/2012
 *
 * create.
 * ----------------------
 */
static int
processbar(PROCESSBAR_T * p)
{
	BOX_T b;
	RECT_T r;
	float temp;

	b.startx = p->x;
	b.starty = p->y;
	b.endx = p->x + p->width;
	b.endy = p->y + 19;
	b.attrib = BOX_RECESSED;
	b.color = COLOR_WINDOW_BACKGROUND;

	gui.box(&b);

	r.x = p->x + 3;
	r.y = p->y + 3;
	r.height = 14;

	temp = p->width - 6;
	temp *= p->percentage;
	temp /= 100.0;

	r.fill_flag = 1;
	r.color = p->color;

	r.width = temp;

	gui.rect(&r);

	return 0;
}
/*
 * Name				: splitter
 * Description		: ---
 * Author			: XiaomaGee.
 *
 * History
 * ----------------------
 * Rev	: 0.00
 * Date	: 03/08/2012
 *
 * create.
 * ----------------------
 */
static int
splitter(SPLITTER_T *s)
{
	int clear = s->attrib & SPLITTER_CLEAR;

	s->attrib &= ~SPLITTER_CLEAR;

	switch (s->attrib & 0xff) {
	case SPLITTER_H | SPLITTER_RECESSED:
		line(s->x, s->y, s->x + s->extend, s->y, clear ? COLOR_WINDOW_BACKGROUND : COLOR_GRAY);
		line(s->x, s->y + 1, s->x + s->extend, s->y + 1, clear ? COLOR_WINDOW_BACKGROUND : COLOR_WHITE);
		break;

	case SPLITTER_H | SPLITTER_RAISED:
		line(s->x, s->y, s->x + s->extend, s->y, clear ? COLOR_WINDOW_BACKGROUND : COLOR_GRAY);
		line(s->x, s->y - 1, s->x + s->extend, s->y - 1, clear ? COLOR_WINDOW_BACKGROUND : COLOR_WHITE);
		break;

	case SPLITTER_V | SPLITTER_RECESSED:
		line(s->x, s->y, s->x, s->y + s->extend, clear ? COLOR_WINDOW_BACKGROUND : COLOR_GRAY);
		line(s->x + 1, s->y, s->x + 1, s->y + s->extend, clear ? COLOR_WINDOW_BACKGROUND : COLOR_WHITE);
		break;

	case SPLITTER_V | SPLITTER_RAISED:
		line(s->x, s->y, s->x, s->y + s->extend, clear ? COLOR_WINDOW_BACKGROUND : COLOR_GRAY);
		line(s->x - 1, s->y, s->x - 1, s->y + s->extend, clear ? COLOR_WINDOW_BACKGROUND : COLOR_WHITE);
		break;
	}

	return 0;
}

/*
 * Name				: check_box
 * Description		: ---
 * Author			: XiaomaGee.
 *
 * History
 * ----------------------
 * Rev	: 0.00
 * Date	: 03/08/2012
 *
 * create.
 * ----------------------
 */
static int
check_box(CHECK_BOX_T * c)
{
	BOX_T b;
	RECT_T r;

	b.startx = c->x;
	b.starty = c->y;
	b.endx = c->x + 10;
	b.endy = c->y + 9;
	b.attrib = BOX_RECESSED;
	b.color = COLOR_WHITE;
	gui.box(&b);

	if (c->attrib == 1) {
		r.x = c->x + 2;
		r.y = c->y + 2;
		r.width = 6;
		r.height = 6;
		r.fill_flag = 1;
		r.color = COLOR_BLACK;

		rect(&r);
	}

	return 0;
}

/*
 * Name				: _draw_circle_8
 * Description		: ---
 * Author			: XiaomaGee.
 *
 * History
 * ----------------------
 * Rev	: 0.00
 * Date	: 03/08/2012
 *
 * create.
 * ----------------------
 */
static
void _draw_circle_8(int xc, int yc, int x, int y, COLOR_T c)
{
	gui.dot(xc + x, yc + y, c);

	gui.dot(xc - x, yc + y, c);

	gui.dot(xc + x, yc - y, c);

	gui.dot(xc - x, yc - y, c);

	gui.dot(xc + y, yc + x, c);

	gui.dot(xc - y, yc + x, c);

	gui.dot(xc + y, yc - x, c);

	gui.dot(xc - y, yc - x, c);
}

/*
 * Name				: circle
 * Description		: ---
 * Author			: XiaomaGee.
 *
 * History
 * ----------------------
 * Rev	: 0.00
 * Date	: 03/08/2012
 *
 * create.
 * ----------------------
 */
static int
circle(int xc, int yc, int r, int fill, COLOR_T c)
{
	int x = 0, y = r, yi, d;

	d = 3 - 2 * r;


	if (fill) {
		// 䣨ʵԲ

		while (x <= y) {
			for (yi = x; yi <= y; yi++)
				_draw_circle_8(xc, yc, x, yi, c);

			if (d < 0) {
				d = d + 4 * x + 6;
			} else {
				d = d + 4 * (x - y) + 10;
				y--;
			}
			x++;
		}
	} else {
		// 䣨Բ

		while (x <= y) {
			_draw_circle_8(xc, yc, x, y, c);
			if (d < 0) {
				d = d + 4 * x + 6;
			} else {
				d = d + 4 * (x - y) + 10;
				y--;
			}
			x++;
		}
	}
	return 0;
}

/*
 * Name				: boxb
 * Description		: ---
 * Author			: XiaomaGee.
 *
 * History
 * ----------------------
 * Rev	: 0.00
 * Date	: 03/08/2012
 *
 * create.
 * ----------------------
 */
static int
boxb(BOX_T * b)
{
	BOX_T b2;

	b2.startx = b->startx + 1;
	b2.starty = b->starty + 1;
	b2.endx = b->endx;
	b2.endy = b->endy - 1;
	b2.attrib = b->attrib;
	b2.color = b->color;
	gui.box(&b2);

	gui.line(b->startx, b->starty, b->startx, b->endy, COLOR_WINDOW_BACKGROUND);
	gui.line(b->startx, b->starty, b->endx, b->starty, COLOR_WINDOW_BACKGROUND);

	gui.line(b->startx, b->endy, b->endx, b->endy, COLOR_BLACK);
	gui.line(b->endx, b->starty, b->endx, b->endy, COLOR_BLACK);

	return 0;
}

/*
 * Name				: show_grid
 * Description		: ---
 * Author			: XiaomaGee.
 *
 * History
 * ----------------------
 * Rev	: 0.00
 * Date	: 03/08/2012
 *
 * create.
 * ----------------------
 */
static int
show_grid(int x, int y, int height)
{
	int i, j;

	lcd_ctl.b.csrmode = CSRMODE_V;
	lcd_ctl(lcd_ctl.w);

	for (j = 0; j < 17; j++) {
		lcd_csrx(x + j);
		lcd_csry(y);
		for (i = 0; i < height; i++) {
			if ((j + i) % 2) LCD_DATA = COLOR_WINDOW_BACKGROUND;
			else LCD_DATA = COLOR_WHITE;
		}
	}

	return 0;
}

/*
 * Name				: scrollbar
 * Description		: ---
 * Author			: XiaomaGee.
 *
 * History
 * ----------------------
 * Rev	: 0.00
 * Date	: 03/08/2012
 *
 * create.
 * ----------------------
 */
static int
scrollbar(SCROLLBAR_T * s)
{
	BOX_T box;
	int i, j;
	int a, b, c;
	float temp;

	if (s->position > s->sum) return -1;

	//TOP
	box.startx = s->x;
	box.starty = s->y;
	box.endx = s->x + 16;
	box.endy = s->y + 16;
	box.attrib = BOX_RAISED;
	box.color = COLOR_WINDOW_BACKGROUND;
	boxb(&box);

	for (i = 3; i >= 0; i--) {
		for (j = i; j < 7 - i; j++) gui.dot(s->x + 5 + j, s->y + 6 + 3 - i, COLOR_BLACK);
	}

	//BUTTOM
	box.startx = s->x;
	box.starty = s->y + s->height - 17;
	box.endx = s->x + 16;
	box.endy = s->y + 16 + s->height - 17;
	box.attrib = BOX_RAISED;
	box.color = COLOR_WINDOW_BACKGROUND;
	boxb(&box);

	for (i = 0; i < 4; i++) {
		for (j = i; j < 7 - i; j++) gui.dot(s->x + 5 + j, s->y + 6 + i + s->height - 17, COLOR_BLACK);
	}

	if (s->sum_per_page >= s->sum) {
		show_grid(s->x, s->y + 17, s->height - 17 * 2);
		return 0;
	}else{
		temp = s->sum_per_page;
		temp /= (s->sum);

		temp *= (s->height - 17 * 2);
		b = temp;

		temp = s->position;
		temp /= (s->sum - 1);

		temp *= (s->height - 17 * 2 - b);
		a = temp;

		c = s->height - 17 * 2 - a - b;

		if (a < 0 || b < 0 || c < 0) return -1;

		show_grid(s->x, s->y + 17, a);

		//bar
		box.startx = s->x;
		box.starty = s->y + 17 + a;
		box.endx = s->x + 16;
		box.endy = s->y + 17 + a + b;
		box.attrib = BOX_RAISED;
		box.color = COLOR_WINDOW_BACKGROUND;
		boxb(&box);

		show_grid(s->x, s->y + 17 + a + b + 1, c - 1);
	}

	return 0;
}
/*
 * Name				: show_picture
 * Description		: ---
 * Author			: XiaomaGee.
 *
 * History
 * ----------------------
 * Rev	: 0.00
 * Date	: 03/08/2012
 *
 * create.
 * ----------------------
 */

static int
show_picture(int x, int y, int width, int height, char * path)
{
//      FIL f;
//      int i,j;
//      char buffer[480*2];
//      unsigned short int *p;
//      unsigned int counter;
//
//      f_open(&f,path,FA_READ);
//
//      lcd_ctl.b.csrmode = CSRMODE_H;
//      lcd_ctl(lcd_ctl.w);

//      for(i=0;i<height;i++){
//                      LCD_ADDX = x;
//                      LCD_ADDY = i+y;
//
//              f_read(&f,buffer,width*2,&counter);
//              p=(unsigned short int *)buffer;
//                      for(j=0;j<width;j++)	LCD_DATA = *p ++;
//
//      }
//      f_close(&f);

	return 0;
}

/*
 * Name				: dialog
 * Description		: ---
 * Author			: XiaomaGee.
 *
 * History
 * ----------------------
 * Rev	: 0.00
 * Date	: 03/08/2012
 *
 * create.
 * ----------------------
 */
/*
   static int
   dialog(const char * fmt,...)
   {
        DIALOG_T d;
        int flag = 1;
        int del_flag = 0;
        va_list arg_ptr;
        char buf[BUFFER_SIZE];

        memset(buf,'\0',sizeof(buf));

        va_start(arg_ptr,fmt);
        vsprintf(buf,fmt,arg_ptr);
        va_end(arg_ptr);

                        while(1){

                                        if(event == KEY_RIGHT || event == KEY_LEFT ){
                                                del_flag = !del_flag;
                                                event = 0;
                                                flag  = 1;
                                        }
                                        if(event == KEY_ENTER){
                                        event = 0;
                                        d.x = 165;
                                        d.y = 110;
                                        d.width = 150;
                                        d.height = 80;
                                        d.color = COLOR_SYSTEM;
                                        d.attrib =  DIALOG_DEL;

                                        _dialog(&d);

                                                break;
                                        }
                                        if(event == KEY_ESC){
                                        event = 0;
                                        d.x = 165;
                                        d.y = 110;
                                        d.width = 150;
                                        d.height = 80;
                                        d.color = COLOR_SYSTEM;
                                        d.attrib =  DIALOG_DEL;

                                        _dialog(&d);

                                                return 0;
                                        }

                                        if(flag){
                                                flag = 0;
                                                d.x = 165;
                                                d.y = 110;
                                                d.width = 150;
                                                d.height = 80;
                                                d.color = COLOR_SYSTEM;
                                                d.attrib = (del_flag?DIALOG_YES:DIALOG_NO) | DIALOG_CREATE;
                                                d.language = sys.language;

                                                d.text = buf;

                                                _dialog(&d);
                                }
                        }

                return del_flag;
   }
 */

/*
 * Name				: dialog
 * Description		: ---
 * Author			: XiaomaGee.
 *
 * History
 * ----------------------
 * Rev	: 0.00
 * Date	: 03/08/2012
 *
 * create.
 * ----------------------
 */
// static int
// _dialog(DIALOG_T *d)
// {
//      BOX_T b;
//      SPLITTER_T sp;
//      int center;
//      STRING_T s;
//      RECT_T r;
//
//      if(d -> attrib & DIALOG_CREATE){
//
//      b.startx = d -> x;
//      b.starty = d -> y;
//      b.endx = d -> x + d -> width;
//      b.endy = d -> y + d -> height;
//      b.color = COLOR_WINDOW_BACKGROUND;
//
//      b.attrib = BOX_RAISED;
//      gui.box(&b);
//
//      sp.x = d -> x;
//      sp.y = d -> y + d -> height - 40;
//
//      sp.extend = d -> width -1;
//      sp.attrib = SPLITTER_H;
//      gui.splitter(&sp);
//
//
//      font._default.single_byte = &fixedsys;
//      font._default.double_byte = &simsun16;
//
//              //yes
//      b.startx =  d -> x +15;
//      b.starty = d -> y + d -> height - 32 - 2;
//      b.endx = d -> x + 15 + 50;
//      b.endy = d -> y + d -> height - 32 - 2 + 28;
//
//      b.color =  ((d -> attrib & 0x1)== DIALOG_YES)? d -> color :COLOR_WINDOW_BACKGROUND ;
//
//      gui.box(&b);


//      b.startx =  d -> x +  d -> width  - 15 - 50;
//      b.starty = d -> y + d -> height - 32 - 2;
//      b.endx =  d -> x +  d -> width  - 15 ;
//      b.endy = d -> y + d -> height - 32 - 2 + 28;
//
//      b.color =  ((d -> attrib & 0x1)== DIALOG_NO)? d -> color :COLOR_WINDOW_BACKGROUND ;
//
//      gui.box(&b);

//      s.x =  d -> x +15  + 17;
//      s.y =  d -> y + d -> height - 32 - 2 + 6;
//      s.color = COLOR_BLACK;
//      s.space.line = 0;
//      s.space.word = 0;
//      s.background_color =((d -> attrib & 0x1)== DIALOG_YES)? d -> color :COLOR_WINDOW_BACKGROUND ;
//      s.inverse = NULL;
//      font.printf(&s,"%s",d -> language == LANGUAGE_EN?"YES":"");
//
//      s.x =  d -> x +  d -> width  - 15 - 50 + 17;
//      s.y =  d -> y + d -> height - 32 - 2 + 6;
//      s.color = COLOR_BLACK;
//      s.space.line = 0;
//      s.space.word = 0;
//      s.background_color =((d -> attrib & 0x1)== DIALOG_NO)? d -> color :COLOR_WINDOW_BACKGROUND ;
//      s.inverse = NULL;
//      font.printf(&s,"%s",d -> language == LANGUAGE_EN?"NO":"");
//
//      font._default.single_byte = &fixedsys;
//      font._default.double_byte = &simsun12;
//
//      s.x =  d -> x + 6;
//      s.y =  d -> y + 6;
//      s.color = COLOR_BLACK;
//      s.space.line = 0;
//      s.space.word = 0;
//      s.background_color =COLOR_WINDOW_BACKGROUND ;
//      s.inverse = NULL;
//      font.printf(&s, d -> text);
//
//      }else {
//
//              r.x = d -> x;
//              r.y = d -> y;
//              r.height = d -> height + 1;
//              r.width = d-> width;
//              r.fill_flag = 1;
//              r.color = COLOR_WINDOW_BACKGROUND;
//
//              gui.rect(&r);
//
//      }
// }
