/*
 * File						: bmp.c
 * Description				: This file is ...
 * Author					: XiaomaGee@Gmail.com
 * Copyright				:
 *
 * History
 ****--------------------
 * Rev						: 0.00
 * Date						: 08/12/2012
 *
 * create.
 ****--------------------
 */

//----------------- Include files ------------------------//
#include <stdio.h>
#include <math.h>

#include "..\include\bmp.h"
#include "..\fatfs\ff.h"
#include "..\include\tft.h"
#include "..\include\color.h"

//----------------- Define -------------------------------//

//----------------- Function Prototype -------------------//
static int get_header_info(char * file, BMP_HEADER_T *);
static int show(char * file, int x, int y);
static int capture(void);

//----------------- Variable -----------------------------//
BMP_T bmp = {
	.header = get_header_info,
	.show = show,
	.capture = capture
};

static FIL f;

//----------------- Function -----------------------------//
/*
 * Name						: get_header_info
 * Description				: ---
 * Author					: XiaomaGee.
 *
 * History
 * -----------------------
 * Rev						: 0.00
 * Date				        : 00/00/2012
 *
 * create.
 * ----------------------
 */
static int
get_header_info(char * file, BMP_HEADER_T *h)
{
	FRESULT res;

	char *p = (char*)h;
	unsigned int counter;

	res = f_open(&f, file, FA_READ);
	if (res != FR_OK) return -1;

	res = f_read(&f, p, sizeof(BMP_HEADER_T), &counter);
	if (res != FR_OK) return -1;

	f_close(&f);

	if (h->type[0] != 'B' || h->type[1] != 'M') return -1;

	return 0;
}

/* Name						: show
 * Description				: ---
 * Author					: XiaomaGee.
 *
 * History
 * -----------------------
 * Rev						: 0.00
 * Date				        : 00/00/2012
 *
 * create.
 * ----------------------
 */
static int
show(char * file, int x, int y)
{
	int i, j;
	int line_byte;
	unsigned int counter;
	char buffer[LINE_PIXEL * 3];
	unsigned short int *p;
	unsigned short int a, b;
	unsigned char color_table[1024];
	unsigned char c;
	BMP_HEADER_T bh;

	if (get_header_info(file, &bh) != 0) return -1;

	f_open(&f, file, FA_READ);

	switch (bh.bit_count) {
	case 1:
		f_lseek(&f, 0x36);
		f_read(&f, color_table, 2 * 4, &counter);

		line_byte = ceil(bh.width / 8);

		if (line_byte % 4) line_byte = ((line_byte / 4) + 1) * 4;

		f_lseek(&f, bh.offset);

		for (i = 0; i < bh.height; i++) {
			f_read(&f, buffer, line_byte, &counter);

			lcd_csrx(x);                        //set x csr
			lcd_csry(y + bh.height - i - 1);    //set y csr

			lcd_ctl.b.csrmode = CSRMODE_H;      //set csr increase direction
			lcd_ctl(lcd_ctl.w);

			tft.write_command(0x0022);          //write data

			for (j = 0; j < bh.width; j++) {
				if (buffer[j / 8] & 1 << (7 - j % 8)) tft.write_data(make_rgb565(color_table[6], color_table[5], color_table[4]));
				else tft.write_data(make_rgb565(color_table[2], color_table[1], color_table[0]));
			}
		}
		break;
	case 4:
		f_lseek(&f, 0x36);
		f_read(&f, color_table, 16 * 4, &counter);

		line_byte = ceil(bh.width / 2.);

		if (line_byte % 4) line_byte = ((line_byte / 4) + 1) * 4;

		f_lseek(&f, bh.offset);

		for (i = 0; i < bh.height; i++) {
			f_read(&f, buffer, line_byte, &counter);

			lcd_csrx(x);                                                            //set x csr
			lcd_csry(y + bh.height - i - 1);                                        //set y csr

			lcd_ctl.b.csrmode = CSRMODE_H;                                          //set csr increase direction
			lcd_ctl(lcd_ctl.w);

			tft.write_command(0x0022);                                              //write data

			for (j = 0; j < bh.width; j++) {
				if (j % 2) c = buffer[j / 2] & 0x0f;
				else c = buffer[j / 2] >> 4;

				tft.write_data(make_rgb565(color_table[c * 4 + 2], color_table[c * 4 + 1], color_table[c * 4 + 0]));
			}
		}
		break;

	case 8:
		f_lseek(&f, 0x36);

		f_read(&f, color_table, 256 * 4, &counter);

		line_byte = bh.width;

		if (line_byte % 4) line_byte = ((line_byte / 4) + 1) * 4;

		f_lseek(&f, bh.offset);

		for (i = 0; i < bh.height; i++) {
			f_read(&f, buffer, line_byte, &counter);

			lcd_csrx(x);                                                                    //set x csr
			lcd_csry(y + bh.height - i - 1);                                                //set y csr

			lcd_ctl.b.csrmode = CSRMODE_H;                                                  //set csr increase direction
			lcd_ctl(lcd_ctl.w);

			tft.write_command(0x0022);                                                      //write data

			for (j = 0; j < bh.width; j++) {
				c = buffer[j];

				tft.write_data(make_rgb565(color_table[c * 4 + 2], color_table[c * 4 + 1], color_table[c * 4 + 0]));
			}
		}
		break;
	case 16:
		line_byte = bh.width * 2;

		if (line_byte % 4) line_byte = ((line_byte / 4) + 1) * 4;

		f_lseek(&f, bh.offset);

		p = (unsigned short int*)buffer;

		for (i = 0; i < bh.height; i++) {
			f_read(&f, buffer, line_byte, &counter);

			lcd_csrx(x);                                                                    //set x csr
			lcd_csry(y + bh.height - i - 1);                                                //set y csr

			lcd_ctl.b.csrmode = CSRMODE_H;                                                   //set csr increase direction
			lcd_ctl(lcd_ctl.w);

			tft.write_command(0x0022);                                                       //write data

			for (j = 0; j < bh.width; j++) {
				switch (bh.compression) {
				case BI_BITFIELDS:                                                            //RGB 565
					tft.write_data(p[j]);
					break;
				case BI_RGB:                                                                  //RGB 555
					a = (p[j] << 1) & 0xf800;

					b = (p[j] << 1) & 0x07c0;

					tft.write_data((p[j] & 0x1f) | a | b);
					break;

				default:
					goto aa;                                                                   //not support
				}
			}
		}

		break;
	case 24:

		line_byte = bh.width * 3;

		if (line_byte % 4) line_byte = ((line_byte / 4) + 1) * 4;

		f_lseek(&f, bh.offset);

		for (i = 0; i < bh.height; i++) {
			f_read(&f, buffer, line_byte, &counter);

			lcd_csrx(x);                                                                    //set x csr
			lcd_csry(y + bh.height - i - 1);                                                //set y csr

			lcd_ctl.b.csrmode = CSRMODE_H;                                                          //set csr increase direction
			lcd_ctl(lcd_ctl.w);

			tft.write_command(0x0022);                                                      //write data

			for (j = 0; j < bh.width; j++) {
				tft.write_data(make_rgb565(buffer[j * 3 + 2], buffer[j * 3 + 1], buffer[j * 3]));
			}
		}

		break;
	case 32:
		goto aa;
	}

	f_close(&f);

	return 0;

 aa:     f_close(&f);

	return -1;
}

/* Name						: capture
 * Description				: ---
 * Author					: XiaomaGee.
 *
 * History
 * -----------------------
 * Rev						: 0.00
 * Date				        : 00/00/2012
 *
 * create.
 * ----------------------
 */

static int judge_fname(void)
{
	int i = 0;
	char buffer[20];

	do {
		sprintf(buffer, "0:/capture/%d.bmp", i);

		if (f_open(&f, buffer, FA_READ) != FR_OK) return i;
		f_close(&f);
		i++;
	} while (1);
}

/* Name						: capture
 * Description				: ---
 * Author					: XiaomaGee.
 *
 * History
 * -----------------------
 * Rev						: 0.00
 * Date				        : 00/00/2012
 *
 * create.
 * ----------------------
 */
static int
capture(void)
{
	int i, j;
	DIR dir;
	char buffer[20];
	char dbuffer[320 * 2];
	unsigned short int *p;
	const char palette[16] = {
		0x00, 0xf8, 0x00, 0x00, //RED
		0xe0, 0x07, 0x00, 0x00, //GREEN
		0x1f, 0x00, 0x00, 0x00, //BLUE
		0x00, 0x00, 0x00, 0x00  //AL
	};
	BMP_HEADER_T bh;
	unsigned int counter;

	//set bmp header infomation
	bh.type[0] = 'B';
	bh.type[1] = 'M';
	bh.file_size = 0x25848;  //320x240 16bit
	bh.reserved = 0;
	bh.offset = 0x46;
	bh.header_info_size = 0x28;

	bh.width = 320;
	bh.height = 240;

	bh.planes = 1;
	bh.bit_count = 16; //16bit
	bh.compression = 0x03; //Bitfields MODE

	bh.image_size = 0x25802; //
	bh.xpels_per_meter = 0;
	bh.ypels_per_meter = 0;
	bh.color_used = 0;
	bh.color_important = 0;

	if (f_opendir(&dir, "0:/capture") != FR_OK) f_mkdir("0:/capture");

	sprintf(buffer, "0:/capture/%d.bmp", judge_fname());

	if (f_open(&f, buffer, FA_CREATE_ALWAYS | FA_WRITE) != FR_OK) return -1;

	f_write(&f, (void*)(&bh), sizeof(BMP_HEADER_T), &counter);
	f_write(&f, palette, sizeof(palette), &counter);

	for (i = 239; i >= 0; i--) {
		p = (unsigned short int*)dbuffer;

		for (j = 0; j < 320; j++) {
			lcd_csrx(j);
			lcd_csry(i);
			get_data();
			*p++ = get_data();
		}
		f_write(&f, dbuffer, 320 * 2, &counter);
	}

	f_close(&f);

	return 0;
}
