/*
 * FILE				: nix.c
 * DESCRIPTION		: This file is iCore nix driver demo.
 * Author			: XiaomaGee@Gmail.com
 * Copyright		:
 *
 * History
 * --------------------
 * Rev	: 0.00
 * Date	: 03/05/2012
 *
 * create.
 * --------------------
 */

//---------------- Include files ------------------------//
#include "..\include\hardware.h"
#include "..\include\nixietube.h"
#include "..\include\systick.h"

#include   <stdio.h>

//---------------- Function Prototype -------------------//
static int initialize(void);

static void display_float(float a);
static void display_int(int a);
static int clear(void);

static int set_led(unsigned char led);

static int scan(void);

static int sel_lvc138(int status);
static int write_hc595(unsigned char dat);

//---------------- Variable -----------------------------//
volatile static unsigned char value[10];

static const unsigned char tab[] = {
	0xc0, 0xf9, 0xa4, 0xb0,
	0x99, 0x92, 0x82, 0xf8,
	0x80, 0x90, 0x88, 0x83,
	0xc6, 0xa1, 0x86, 0x8e, //0~f
	0xff, 0xbf
};

NIX_T nix = {
	.initialize = initialize,
	.display_float = display_float,
	.display_int = display_int,
	.set_led = set_led,
	.scan = scan,
	.clear = clear
};

//-----------------Function------------------------------//
/*
 * Name			: initialize_lvc138
 * Description	: ---
 * Author		: XiaomaGee.
 *
 * History
 * ----------------------
 * Rev	: 0.00
 * Date	: 03/05/2012
 *
 * create.
 * ----------------------
 */
static int
initialize_lvc138(void)
{
	NIX_SEL0_OUTPUT;
	NIX_SEL1_OUTPUT;
	NIX_SEL2_OUTPUT;

	return 0;
}

/*
 * Name				: initialize_hc595
 * Description		: ---
 * Author			: XiaomaGee.
 *
 * History
 * ----------------------
 * Rev	: 0.00
 * Date	: 03/05/2012
 *
 * create.
 * ----------------------
 */
static int
initialize_hc595(void)
{
	NIX_B0_OUTPUT;
	NIX_B1_OUTPUT;
	NIX_B2_OUTPUT;

	NIX_B0_ON;
	NIX_B1_ON;
	NIX_B2_ON;

	return 0;
}

/*
 * Name				: sel_lvc138
 * Description		: ---
 * Author			: XiaomaGee.
 *
 * History
 * ----------------------
 * Rev	: 0.00
 * Date	: 03/05/2012
 *
 * create.
 * ----------------------
 */
static int
sel_lvc138(int status)
{
	GPIOA->ODR.W |= 0X00E0;

	GPIOA->ODR.W &= (status << 5);

	return 0;
}

/*
 * Name				: write_hc595
 * Description		: ---
 * Author			: XiaomaGee.
 *
 * History
 * ----------------------
 * Rev	: 0.00
 * Date	: 03/05/2012
 *
 * create.
 * ----------------------
 */
static int
write_hc595(unsigned char dat)
{
	int i;

	for (i = 0; i < 8; i++) {
		if (dat & 0x80) {
			NIX_B0_ON;
		}else {
			NIX_B0_OFF;
		}
		NIX_B2_OFF;
		NIX_B2_ON;

		dat <<= 1;
	}
	NIX_B1_OFF;
	NIX_B1_ON;

	return 0;
}

/*
 * Name				: initialize
 * Description		: ---
 * Author			: XiaomaGee.
 *
 * History
 * ----------------------
 * Rev	: 0.00
 * Date	: 03/05/2012
 *
 * create.
 * ----------------------
 */
static int
initialize(void)
{
	initialize_lvc138();
	initialize_hc595();

	clear();

	return 0;
}

/*
 * Name				: display_float
 * Description		: ---
 * Author			: XiaomaGee.
 *
 * History
 * ----------------------
 * Rev	: 0.00
 * Date	: 03/05/2012
 *
 * create.
 * ----------------------
 */
static
void display_float(float a)
{
	char buf[10];

	if (a < 0) {
		a = 0 - a;
		value[0] = 17;                                  //ǸĻλܣʾ-,
	}else value[0] = 16;                            //Ļʾո

	if (a < 10) {
		sprintf(buf, "%.2f", a);
		value[0] += 100;                        //λʾС㣨
		value[1] = buf[0] - '0';                //value[1] = buf[0] - '0'+100ʾλС
		value[2] = buf[2] - '0';
		value[3] = buf[3] - '0';
	}else if (a < 100) {
		sprintf(buf, "%.1f", a);
		value[1] = buf[0] - '0';
		value[2] = buf[1] - '0' + 100;  //һλʾС㣨
		value[3] = buf[3] - '0';
	}else if (a < 1000) {
		sprintf(buf, "%.0f", a);
		value[1] = buf[0] - '0';
		value[2] = buf[1] - '0';
		value[3] = buf[2] - '0';
	} else {
		value[0] = 0xe;                                 //ʾĻE
		value[1] = 16;
		value[2] = 16;
		value[3] = 16;

		return;
	}
}

/*
 * Name				: display_int
 * Description		: ---
 * Author			: XiaomaGee.
 *
 * History
 * ----------------------
 * Rev	: 0.00
 * Date	: 03/05/2012
 *
 * create.
 * ----------------------
 */
static
void display_int(int a)
{
	char buf[10];

	if (a < 0) {
		a = 0 - a;

		if (a < 10) {
			sprintf(buf, "%d", a);
			value[0] = 16;
			value[1] = 16;                          // ' '
			value[2] = 17;                          // '-'
			value[3] = buf[0] - '0';
		}else if (a < 100) {
			sprintf(buf, "%d", a);
			value[0] = 16;
			value[1] = 17;                          // ' '
			value[2] = buf[0] - '0';
			value[3] = buf[1] - '0';
		}else if (a < 1000) {
			sprintf(buf, "%d", a);
			value[0] = 17;
			value[1] = buf[0] - '0';
			value[2] = buf[1] - '0';
			value[3] = buf[2] - '0';
		}else {
			value[0] = 0xe;
			value[1] = 16;
			value[2] = 16;
			value[3] = 16;
		}
	}else {                                                         // >=0
		if (a < 10) {
			sprintf(buf, "%d", a);
			value[0] = 16;
			value[1] = 16;                          // ' '
			value[2] = 16;                          // ''
			value[3] = buf[0] - '0';
		}else if (a < 100) {
			sprintf(buf, "%d", a);
			value[0] = 16;
			value[1] = 16;                          // ' '
			value[2] = buf[0] - '0';
			value[3] = buf[1] - '0';
		}else if (a < 1000) {
			sprintf(buf, "%d", a);
			value[0] = 16;
			value[1] = buf[0] - '0';
			value[2] = buf[1] - '0';
			value[3] = buf[2] - '0';
		}else if (a < 10000) {
			sprintf(buf, "%d", a);
			value[0] = buf[0] - '0';
			value[1] = buf[1] - '0';
			value[2] = buf[2] - '0';
			value[3] = buf[3] - '0';
		}else {
			value[0] = 0xe;
			value[1] = 16;
			value[2] = 16;
			value[3] = 16;
		}
	}
}

/*
 * Name				: set_led
 * Description		: ---
 * Author			: XiaomaGee.
 *
 * History
 * ----------------------
 * Rev	: 0.00
 * Date	: 03/05/2012
 *
 * create.
 * ----------------------
 */
static
int set_led(unsigned char led)
{
	value[4] = ~led;

	return 0;
}

/*
 * Name				: scan
 * Description		: ---
 * Author			: XiaomaGee.
 *
 * History
 * ----------------------
 * Rev	: 0.00
 * Date	: 03/05/2012
 *
 * create.
 * ----------------------
 */
static
int scan(void)
{
	static int ct_nix = 0;
	static int key_code = 0;
	static int key_code_temp = 0;

	sel_lvc138(7);

	if (ct_nix == 0) write_hc595(value[4]);  //ct_nix == 0ѡͨLED
	else write_hc595(value[4 - ct_nix] > 99 ? tab[value[4 - ct_nix] - 100] & 0x7f : tab[value[4 - ct_nix]]);
	//ct_nixΪ0value[4-ct_nix] > 99ʱ֤С㣬ִtab[value[4-ct_nix] - 100] & 0x7fִtab[value[4-ct_nix]
	sel_lvc138(ct_nix);                              //ѡͨӦİ

	if (KEY_IN == 0) {
		key_code_temp += (1 << ct_nix);  //KEY_IN == 01ct_nixλkey_code_tempֵӺٽkey_code_temp
	}

	ct_nix++;

	if (ct_nix > 4) {                         //ct_nix > 4򰴼ȫɨһ
		ct_nix = 0;                      //ct_nix0
		key_code = key_code_temp;        //key_code_tempֵkey_code
		key_code_temp = 0;               //key_code_temp0
	}

	return key_code;                         //key_codeֵ
}

/*
 * Name				: clear
 * Description		: ---
 * Author			: XiaomaGee.
 *
 * History
 * ----------------------
 * Rev	: 0.00
 * Date	: 03/05/2012
 *
 * create.
 * ----------------------
 */
static
int clear(void)
{
	int i;

	for (i = 0; i < 4; i++) {
		value[i] = 16;
	}
	value[4] = 0xff;

	return 0;
}
