/*
 * FILE				: spi.c
 * DESCRIPTION		: This file is main files.
 * Author			: XiaomaGee@Gmail.com
 * Copyright		:
 *
 * History
 * --------------------
 * Rev	: 0.00
 * Date	: 03/05/2012
 *
 * create.
 * --------------------
 */

//-----------------Include files-------------------------//
#include "..\include\hardware.h"
#include "..\include\spi.h"
#include <stdio.h>

//---------------- Function Prototype ------------------//
static int initialize_spi2(void);
static int write_spi2(int number, unsigned char * buf);
static unsigned char send_data_spi2(unsigned char data);

static int initialize_spi1(void);
static int write_spi1(int number, unsigned char * buf);
static unsigned char send_data_spi1(unsigned char data);

//-----------------Variable-----------------------------//
SYS_SPI_T spi2 = {
	.initialize = initialize_spi2,
	.write = write_spi2,
	.send_data = send_data_spi2
};

SYS_SPI_T spi1 = {
	.initialize = initialize_spi1,
	.write = write_spi1,
	.send_data = send_data_spi1
};

//-----------------Function-----------------------------//
/*
 * FILE				: initialize_spi1
 * DESCRIPTION		: This file is main files.
 * Author			: XiaomaGee@Gmail.com
 * Copyright		:
 *
 * History
 * --------------------
 * Rev	: 0.00
 * Date	: 03/05/2012
 *
 * create.
 * --------------------
 */
static int initialize_spi1(void)
{
	RCC->APB2ENR.B.SPI1EN = 1;

	AFIO->MAPR.B.SPI1_REMAP = 1; //spi1 remap

	AFIO->MAPR.B.SWJ_CFG = 2;

	GPIOB->CRL.B.MODE7 = 0x3;       //NSS 50M
	GPIOB->CRL.B.CNF7 = 0x0;         //10ùģʽ

	GPIOB->CRL.B.MODE3 = 0x3;       //CSK 50M
	GPIOB->CRL.B.CNF3 = 0x2;        //10ùģʽ

	GPIOB->CRL.B.MODE5 = 0x3;       //MOSI 50M
	GPIOB->CRL.B.CNF5 = 0x2;        //10ùģʽ

	GPIOB->CRL.B.MODE4 = 0x0;       //MISO
	GPIOB->CRL.B.CNF4 = 0x2;        //01ģʽ(λ״̬)        //10/ģʽ

	SPI1->CR1.B.BR = 0;             //000 fPCLK/2//001 fPCLK/4//010 fPCLK/8//011 fPCLK/16
	//100 fPCLK/32	//101 fPCLK/64  //110 fPCLK/128	36M/128	//111 fPCLK/256
	SPI1->CR1.B.BIDIMODE = 0;       //0ѡ˫˫ģʽ
	SPI1->CR1.B.CPOL = 1;
	SPI1->CR1.B.CPHA = 1;
	SPI1->CR1.B.DFF = 0;            //1ʹ16λ֡ʽз/գ
	//0ʹ8λ֡ʽз/գ
	SPI1->CR1.B.LSBFIRST = 0;       //0ȷMSB
	SPI1->CR2.B.SSOE = 1;           //1豸ʱģʽSS
	SPI1->CR1.B.MSTR = 1;           //1Ϊ豸

	SPI1->CR1.B.SSM = 1;
	SPI1->CR1.B.SSI = 1;

	SPI1->CR1.B.SPE = 1;            //1SPI豸

	SPI1_CS_ON;

	return 0;
}

/*
 * FILE				: initialize_spi2
 * DESCRIPTION		: This file is main files.
 * Author			: XiaomaGee@Gmail.com
 * Copyright		:
 *
 * History
 * --------------------
 * Rev	: 0.00
 * Date	: 03/05/2012
 *
 * create.
 * --------------------
 */
static int initialize_spi2(void)
{
	RCC->APB1ENR.B.SPI2EN = 1;

	GPIOB->CRH.B.MODE12 = 0x3;       //NSS 50M
	GPIOB->CRH.B.CNF12 = 0x0;         //10ùģʽ
	GPIOB->CRH.B.MODE13 = 0x3;       //CSK 50M
	GPIOB->CRH.B.CNF13 = 0x2;        //10ùģʽ
	GPIOB->CRH.B.MODE15 = 0x3;       //MOSI 50M
	GPIOB->CRH.B.CNF15 = 0x2;        //10ùģʽ

	GPIOB->CRH.B.MODE14 = 0x0;       //MISO
	GPIOB->CRH.B.CNF14 = 0x1;        //01ģʽ(λ״̬) //10/ģʽ

	SPI2->CR1.B.BR = 4;              //000 fPCLK/2//001 fPCLK/4//010 fPCLK/8//011 fPCLK/16  //100 fPCLK/32
	//101 fPCLK/64   //110 fPCLK/128	36M/128	 //111 fPCLK/256
	SPI2->CR1.B.BIDIMODE = 0;         //0ѡ˫˫ģʽ
	SPI2->CR1.B.CPOL = 0;
	SPI2->CR1.B.CPHA = 0;
	SPI2->CR1.B.DFF = 0;             //1ʹ16λ֡ʽз/գ
	//0ʹ8λ֡ʽз/գ
	SPI2->CR1.B.LSBFIRST = 0;        //0ȷMSB
	SPI2->CR2.B.SSOE = 1;            //1豸ʱģʽSS
	SPI2->CR1.B.MSTR = 1;            //1Ϊ豸

	SPI2->CR1.B.SSM = 1;
	SPI2->CR1.B.SSI = 1;

	SPI2->CR1.B.SPE = 1;              //1SPI豸

	SPI2_CS_ON;

	return 0;
}

/*
 * FILE				: send_data_spi
 * DESCRIPTION		: This file is main files.
 * Author			: XiaomaGee@Gmail.com
 * Copyright		:
 *
 * History
 * --------------------
 * Rev	: 0.00
 * Date	: 03/05/2012
 *
 * create.
 * --------------------
 */
static unsigned char send_data_spi2(unsigned char data)
{
	while (!(SPI2->SR.B.TXE == 1)) ;

	SPI2->DR = data;

	while (!(SPI2->SR.B.RXNE == 1)) ;

	return SPI2->DR;
}

static unsigned char send_data_spi1(unsigned char data)
{
	while (!(SPI1->SR.B.TXE == 1)) ;

	SPI1->DR = data;

	while (!(SPI1->SR.B.RXNE == 1)) ;

	return SPI1->DR;
}

/*
 * FILE				: write_spi
 * DESCRIPTION		: This file is main files.
 * Author			: XiaomaGee@Gmail.com
 * Copyright		:
 *
 * History
 * --------------------
 * Rev	: 0.00
 * Date	: 03/05/2012
 *
 * create.
 * --------------------
 */
static int write_spi1(int number, unsigned char * buf)
{
	int i;

	SPI1_CS_OFF;

	for (i = 0; i < number; i++) buf[i] = send_data_spi1(buf[i]);

	SPI1_CS_ON;

	return 0;
}

static int write_spi2(int number, unsigned char * buf)
{
	int i;

	SPI2_CS_OFF;

	for (i = 0; i < number; i++) buf[i] = send_data_spi2(buf[i]);

	SPI2_CS_ON;

	return 0;
}
