/*
 * FILE				: tft.c
 * DESCRIPTION		: This file is main files.
 * Author			: XiaomaGee@Gmail.com
 * Copyright		:
 *
 * History
 * --------------------
 * Rev	: 0.00
 * Date	: 03/05/2012
 *
 * create.
 * --------------------
 */

//-----------------Include files-------------------------//
#include "..\include\stm32f10x_reg.h"
#include "..\include\hardware.h"
#include "..\include\fsmc.h"
#include "..\include\systick.h"
#include "..\include\tft.h"

//---------------- Function Prototype -------------------//
static int initialize(void);
static int write_command(unsigned short int command);
static int write_data(unsigned short int data);
static int fill(unsigned short int color);
static int set_point(unsigned short int x, unsigned short int y, unsigned short int color);
static int lcd_ctl_set(unsigned short int ctl_w);
static unsigned short int  get_point(unsigned short int x, unsigned short int y);
//---------------- Variable -----------------------------//
TFT_T tft = {
	.initialize = initialize,
	.cls = fill,
	.set_point = set_point,
	.get_point = get_point,
	.write_command = write_command,
	.write_data = write_data,
	.lcd_ctl = lcd_ctl_set,
};

LCD_CTL_T lcd_ctl;

//-----------------Function------------------------------//
/*
 * Name				: delay
 * Description		: ---
 * Author			: XiaomaGee.
 *
 * History
 * ----------------------
 * Rev	: 0.00
 * Date	: 03/05/2012
 *
 * create.
 * ----------------------
 */
static void delay(int ct)
{
	int i, j;

	for (i = 0; i < ct; i++) {
		for (j = 0; j < 10000; j++) ;
	}
}

/*
 * Name				: lcd
 * Description		: ---
 * Author			: XiaomaGee.
 *
 * History
 * ----------------------
 * Rev	: 0.00
 * Date	: 03/05/2012
 *
 * create.
 * ----------------------
 */
void lcd_data(unsigned short int x)
{
	tft.write_command(0x0022);
	tft.write_data(x);
}

void lcd_csrx(unsigned short int x)
{
	tft.write_command(0x0021);
	tft.write_data(319 - x);
}

void lcd_csry(unsigned short int x)
{
	tft.write_command(0x0020);
	tft.write_data(x);
}

/*
 * Name				: LCD_RegInit
 * Description		: ---
 * Author			: XiaomaGee.
 *
 * History
 * ----------------------
 * Rev	: 0.00
 * Date	: 03/05/2012
 *
 * create.
 * ----------------------
 */
void LCD_RegInit(unsigned short int x, unsigned short int y)
{
	write_command(x);
	write_data(y);
}

/*
 * Name				: int get_data
 * Description		: ---
 * Author			: XiaomaGee.
 *
 * History
 * ----------------------
 * Rev	: 0.00
 * Date	: 03/05/2012
 *
 * create.
 * ----------------------
 */
unsigned short int get_data(void)
{
	tft.write_command(0x0022);
	return LCD_DATA;
}

/*
 * Name				: initialize
 * Description		: ---
 * Author			: XiaomaGee.
 *
 * History
 * ----------------------
 * Rev	: 0.00
 * Date	: 03/05/2012
 *
 * create.
 * ----------------------
 */
static int initialize(void)
{
	TFTBL_OUTPUT;
	TFTBL_ON;

	TFTRST_OUTPUT;
	TFTRST_OFF;
	delay(10);
	TFTRST_ON;

//-----Normal Setting-----------------------
	LCD_RegInit(0x0000, 0x0001);
	LCD_RegInit(0x0001, 0x0100);
	LCD_RegInit(0x0002, 0x0700);
	LCD_RegInit(0x0003, 0x0018);
	LCD_RegInit(0x0004, 0x0000);
	LCD_RegInit(0x0008, 0x0202);
	LCD_RegInit(0x0009, 0x0000);
	LCD_RegInit(0x000A, 0x0000);
	LCD_RegInit(0x000C, 0x0000);
	LCD_RegInit(0x000D, 0x0000);
	LCD_RegInit(0x000F, 0x0000);
//-----Power On sequence-----------------------
	LCD_RegInit(0x0010, 0x0000);
	LCD_RegInit(0x0011, 0x0000);
	LCD_RegInit(0x0012, 0x0000);
	LCD_RegInit(0x0013, 0x0000);
	delay(150);
	LCD_RegInit(0x0010, 0x17B0);
	LCD_RegInit(0x0011, 0x0007);
	delay(50);
	LCD_RegInit(0x0012, 0x013A);
	delay(50);
	LCD_RegInit(0x0013, 0x1a00);    //1a00
	LCD_RegInit(0x0029, 0x0014);
	delay(50);
//-----Gamma control-----------------------
	LCD_RegInit(0x0030, 0x0000);
	LCD_RegInit(0x0031, 0x0707);
	LCD_RegInit(0x0032, 0x0700);
	LCD_RegInit(0x0035, 0x0004);
	LCD_RegInit(0x0036, 0x1f00);
	LCD_RegInit(0x0037, 0x0004);
	LCD_RegInit(0x0038, 0x0707);
	LCD_RegInit(0x0039, 0x0007);
	LCD_RegInit(0x003C, 0x0400);
	LCD_RegInit(0x003D, 0x1f00);
//-----Set RAM area-----------------------
	LCD_RegInit(0x0050, 0x0000);
	LCD_RegInit(0x0051, 0x00EF);
	LCD_RegInit(0x0052, 0x0000);
	LCD_RegInit(0x0055, 0x013F);
	LCD_RegInit(0x0060, 0x2700);
	LCD_RegInit(0x0061, 0x0001); //0001
	LCD_RegInit(0x006A, 0x0000);
	LCD_RegInit(0x0021, 0x0000);
	LCD_RegInit(0x0020, 0x0000);
//-----Partial Display Control------------
	LCD_RegInit(0x0080, 0x0000);
	LCD_RegInit(0x0081, 0x0000);
	LCD_RegInit(0x0082, 0x0000);
	LCD_RegInit(0x0083, 0x0000);
	LCD_RegInit(0x0084, 0x0000);
	LCD_RegInit(0x0085, 0x0000);
//-----Panel Control----------------------
	LCD_RegInit(0x0090, 0x0010);
	LCD_RegInit(0x0092, 0x0000);
	LCD_RegInit(0x0093, 0x0003);
	LCD_RegInit(0x0095, 0x0110);
	LCD_RegInit(0x0097, 0x0000);
	LCD_RegInit(0x0098, 0x0000);
//-----Display on-----------------------
	LCD_RegInit(0x0007, 0x0173);

	delay(50);

	return 0;
}

/*
 * Name				: write_command
 * Description		: ---
 * Author			: XiaomaGee.
 *
 * History
 * ----------------------
 * Rev	: 0.00
 * Date	: 03/05/2012
 *
 * create.
 * ----------------------
 */
static int write_command(unsigned short int command)
{
	LCD_COMMAND = command;

	return 0;
}

/*
 * Name				: write_data
 * Description		: ---
 * Author			: XiaomaGee.
 *
 * History
 * ----------------------
 * Rev	: 0.00
 * Date	: 03/05/2012
 *
 * create.
 * ----------------------
 */
static int write_data(unsigned short int data)
{
	LCD_DATA = data;

	return 0;
}

/*
 * Name				: set_point
 * Description		: ---
 * Author			: XiaomaGee.
 *
 * History
 * ----------------------
 * Rev	: 0.00
 * Date	: 03/05/2012
 *
 * create.
 * ----------------------
 */
static int set_point(unsigned short int x, unsigned short int y, unsigned short int color)
{
	write_command(0x0020); write_data(y);
	write_command(0x0021); write_data(319 - x);

	write_command(0x0022); write_data(color);

	return 0;
}

/*
 * Name				: set_point
 * Description		: ---
 * Author			: XiaomaGee.
 *
 * History
 * ----------------------
 * Rev	: 0.00
 * Date	: 03/05/2012
 *
 * create.
 * ----------------------
 */
static unsigned short int get_point(unsigned short int x, unsigned short int y)
{
	lcd_csrx(x);
	lcd_csry(y);
	get_data();

	return get_data();
}


/*
 * Name				: fill
 * Description		: ---
 * Author			: XiaomaGee.
 *
 * History
 * ----------------------
 * Rev	: 0.00
 * Date	: 03/05/2012
 *
 * create.
 * ----------------------
 */
static int fill(unsigned short int color)
{
	unsigned long int i;

	write_command(0x0020); //y ˮƽGRAMַ
	write_data(0);
	write_command(0x0021); //x ֱGRAMַ
	write_data(319);
	write_command(0x0022); //дdataGRAM

	for (i = 0; i < 320 * 240; i++) {
		write_data(color);
	}
	return 0;
}

/*
 * Name				: lcd_ctl_se
 * Description		: ---
 * Author			: XiaomaGee.
 *
 * History
 * ----------------------
 * Rev	: 0.00
 * Date	: 03/05/2012
 *
 * create.
 * ----------------------
 */
static int lcd_ctl_set(unsigned short int ctl_w)
{
	if ((ctl_w & 0x0040) == 0) {
		write_command(0x0003);
		write_data(0x0018);
	}else {
		write_command(0x0003);
		write_data(0x0010);
	}
	return 0;
}
