/*
 * FILE			: usart.c
 * DESCRIPTION	: This file is iCore usart driver.
 * Author		: XiaomaGee@Gmail.com
 * Copyright	:
 *
 * History
 * --------------------
 * Rev			: 0.00
 * Date			: 03/05/2012
 *
 * create.
 * --------------------
 */
//-----------------Include files-------------------------//

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <math.h>
#include <stdarg.h>
#include "..\include\stm32f10x_reg.h"
#include "..\include\usart.h"
#include "..\include\hardware.h"

//-----------------Variable------------------------------//
static int send_string_to_usart1(char * str);
static int send_byte_to_usart1(char data);
static int initialize_usart1(int baudrate);
static int my_printf1(const char * fmt, ...);
static int send_string_to_usart2(char * str);
static int send_byte_to_usart2(char data);
static int initialize_usart2(int baudrate);
static int my_printf_uart2(const char * fmt, ...);

UART_T usart1 = {
	.receive_ok_flag = 0,
	.receive_count = 0,
	.send_byte = send_byte_to_usart1,
	.send_string = send_string_to_usart1,
	.initialize = initialize_usart1,
	.printf = my_printf1
};

UART_T usart2 = {
	.receive_ok_flag = 0,
	.receive_count = 0,
	.send_byte = send_byte_to_usart2,
	.send_string = send_string_to_usart2,
	.initialize = initialize_usart2,
	.printf = my_printf_uart2
};

//-----------------Function------------------------------//
/*
 * Name			: initialize
 * Description	: ---
 * Author		: XiaomaGee.
 *
 * History
 * ----------------------
 * Rev			: 0.00
 * Date			: 03/05/2012
 *
 * create.
 * ----------------------
 */
static int initialize_usart1(int baudrate)
{
	float temp;
	int temp2;

	RCC->APB2ENR.B.IOPAEN = 1;              //Enable GPIOA clocks
	RCC->APB2ENR.B.USART1EN = 1;
	GPIOA->CRH.B.MODE9 = 3;                     // Configure USART1 Tx (PA.09) as alternate function push-pull
	GPIOA->CRH.B.CNF9 = 2;
	GPIOA->CRH.B.MODE10 = 0;                        // Configure USART1 Rx (PA.10) as input floating
	GPIOA->CRH.B.CNF10 = 1;

	temp = CPU_CLK / (16 * baudrate);
	temp2 = floor(temp);
	temp = temp - temp2;
	temp *= 16;

	USART1->BRR.B.DIV_Fraction = temp;  //DIV_Fraction[3:0]USARTDIV
	//4λUSARTƵ(USARTDIV)С֡
	// 15 115200
	USART1->BRR.B.DIV_Mantissa = temp2;     //DIV_Mantissa[11:0]USARTDIVС
	//12λUSARTƵ(USARTDIV)֡
	//113 115200
	USART1->CR1.B.M = 0;                    //0һʼλ8λnֹͣλ
	USART1->CR2.B.STOP = 0;                         //001ֹͣλ
	USART1->CR1.B.PCE = 0;                          //0УƱֹ
	USART1->CR3.B.RTSE = 0;                         //0RTSӲƱֹ
	USART1->CR3.B.CTSE = 0;                         //0CTSӲƱֹ
	USART1->CR1.B.TE = 1;                   //1ͱʹܡ
	USART1->CR1.B.RE = 1;                   //1ܱʹܡ
	USART1->CR2.B.CLKEN = 0;                        //1SCLKűʹܡ
	USART1->CR2.B.CPOL = 0;                         //0߿ʱSCLKϱֵ͵ƽ
	USART1->CR2.B.CPHA = 1;                         //1ʱӵڶؽݲ
	USART1->CR2.B.LBCL = 0;                         //0һλݵʱ岻SCLK

	USART1->CR1.B.PEIE = 1;                         //1USART_SRеPEΪ1ʱUSARTжϡ
	USART1->CR1.B.RXNEIE = 1;                       //1USART_SRеORERXNEΪ1ʱUSARTжϡ

	USART1->CR1.B.UE = 1;                   //1USARTģʹܡ

	return 0;
}

static int initialize_usart2(int baudrate)
{
	float temp;
	int temp2;

	RCC->APB2ENR.B.IOPAEN = 1;                      //Enable GPIOA clocks
	RCC->APB1ENR.B.USART2EN = 1;

	/* USART1 configuration ------------------------------------------------------*/
	/* USART1 configured as follow:
	    - BaudRate = 115200 baud
	    - Word Length = 8 Bits
	    - One Stop Bit
	    - No parity
	    - Hardware flow control disabled (RTS and CTS signals)
	    - Receive and transmit enabled
	    - USART Clock disabled
	    - USART CPOL: Clock is active low
	    - USART CPHA: Data is captured on the middle
	    - USART LastBit: The clock pulse of the last data bit is not output to
	                     the SCLK pin
	 */
	GPIOA->CRL.B.MODE2 = 3;                           // Configure USART1 Tx (PA.2) as alternate function push-pull
	GPIOA->CRL.B.CNF2 = 2;
	GPIOA->CRL.B.MODE3 = 0;                           // Configure USART1 Rx (PA.3) as input floating
	GPIOA->CRL.B.CNF3 = 1;

	temp = CPU_CLK / (16 * baudrate);
	temp /= 2;                                                      //APB1= 1/2 CPU CLK;
	temp2 = floor(temp);
	temp = temp - temp2;
	temp *= 16;

	USART2->BRR.B.DIV_Fraction = temp;        //DIV_Fraction[3:0]USARTDIV
	//4λUSARTƵ(USARTDIV)С֡
	// 15 115200
	USART2->BRR.B.DIV_Mantissa = temp2;       //DIV_Mantissa[11:0]USARTDIVС
	//12λUSARTƵ(USARTDIV)֡
	//113 115200
	USART2->CR1.B.M = 0;                                      //0һʼλ8λnֹͣλ
	USART2->CR2.B.STOP = 0;                           //001ֹͣλ
	USART2->CR1.B.PCE = 0;                            //0УƱֹ
	USART2->CR3.B.RTSE = 0;                           //0RTSӲƱֹ
	USART2->CR3.B.CTSE = 0;                           //0CTSӲƱֹ
	USART2->CR1.B.TE = 1;                                     //1ͱʹܡ
	USART2->CR1.B.RE = 1;                                     //1ܱʹܡ
	USART2->CR2.B.CLKEN = 0;                          //1SCLKűʹܡ
	USART2->CR2.B.CPOL = 0;                           //0߿ʱSCLKϱֵ͵ƽ
	USART2->CR2.B.CPHA = 1;                           //1ʱӵڶؽݲ
	USART2->CR2.B.LBCL = 0;                           //0һλݵʱ岻SCLK
	USART2->CR1.B.PEIE = 1;                           //1USART_SRеPEΪ1ʱUSARTжϡ
	USART2->CR1.B.RXNEIE = 1;                         //1USART_SRеORERXNEΪ1ʱUSARTжϡ
	USART2->CR1.B.UE = 1;                                     //1USARTģʹܡ

	RS485_OUTPUT;

	RS485_R;

	return 0;
}

/*
 * Name			: initialize
 * Description	: ---
 * Author		: XiaomaGee.
 *
 * History
 * ----------------------
 * Rev			: 0.00
 * Date			: 03/05/2012
 *
 * create.
 * ----------------------
 */
static int
send_byte_to_usart1(char data)
{
	USART1->DR.W = data;
	while (!(USART1->SR.B.TC == 1)) ;

	return 0;
}

static int
send_byte_to_usart2(char data)
{
	RS485_T;

	USART2->DR.W = data;

	while (!(USART2->SR.B.TC == 1)) ;

	RS485_R;

	return 0;
}
/*-------------------------------------------------------
   NAME       : send_string_to_usart1
   ENTRY  TYPE: void
   RETURN TYPE: void
   PURPOSE    : ---
   VERSION    : 0.0.0
   DESCRIPTION:
   CREATE DATE: ---
   LOGS       :
   ---------------------------------------------------------*/
static int
send_string_to_usart1(char * str)
{
	while (*str != '\0') {
		while (!(USART1->SR.B.TC == 1)) ;
		USART1->DR.W = (*str++);
	}

	return 0;
}

static int
send_string_to_usart2(char * str)
{
	RS485_T;

	while (*str != '\0') {
		while (!(USART2->SR.B.TC == 1)) ;
		USART2->DR.W = (*str++);
	}
	while (!(USART2->SR.B.TC == 1)) ;
	RS485_R;

	return 0;
}

/*
 * Name			: initialize
 * Description	: ---
 * Author		: XiaomaGee.
 *
 * History
 * ----------------------
 * Rev			: 0.00
 * Date			: 03/05/2012
 *
 * create.
 * ----------------------
 */
int
USART1_IRQHandler(void)
{
	while (USART1->SR.B.RXNE == 0) ;
	usart1.receive_buffer[usart1.receive_count++] = USART1->DR.W;
	if (usart1.receive_buffer[usart1.receive_count - 1] == '\r') {
		usart1.receive_count = 0;
		usart1.receive_ok_flag = 1;
	}

	return 0;
}

int
USART2_IRQHandler(void)
{
	while (USART2->SR.B.RXNE == 0) ;
	usart2.receive_buffer[usart2.receive_count++] = USART2->DR.W;

	if (usart2.receive_buffer[usart2.receive_count - 1] == '\n') {
		usart2.receive_buffer[usart2.receive_count - 1] = 0;
		usart2.receive_count = 0;
		usart2.receive_ok_flag = 1;
	}

	return 0;
}

/*
 * Name			: printf1
 * Description	: ---
 * Author		: XiaomaGee.
 *
 * History
 * ----------------------
 * Rev			: 0.00
 * Date			: 03/05/2012
 *
 * create.
 * ----------------------
 */
static int
my_printf1(const char * fmt, ...)
{
	va_list arg_ptr;
	char buf[BUFFER_SIZE];

	memset(buf, '\0', sizeof(buf));

	va_start(arg_ptr, fmt);
	vsprintf(buf, fmt, arg_ptr);
	va_end(arg_ptr);

	send_string_to_usart1(buf);

	return 0;
}

static int
my_printf_uart2(const char * fmt, ...)
{
	va_list arg_ptr;
	char buf[BUFFER_SIZE];

	memset(buf, '\0', sizeof(buf));

	va_start(arg_ptr, fmt);
	vsprintf(buf, fmt, arg_ptr);
	va_end(arg_ptr);

	send_string_to_usart2(buf);

	return 0;
}
